CON

  _clkmode      = xtal1 + pll16x
  _xinfreq      = 5_000_000

  GPS_RAW_PIN   = 6             
  GPS_RX_PIN    = 7

    ' US time zones. Pass these into the start function to set the time zone.         
  EASTERN_TIME            = -5                    ' Eastern Standard Time
  CENTRAL_TIME            = -6                    ' Central Standard Time 
  MOUNTAIN_TIME           = -7                    ' Mountain Standard Time
  PACIFIC_TIME            = -8                    ' Pacific Standard Time
  ALASKAN_TIME            = -9                    ' Alaskan Standard Time
  HAWAIIAN_TIME           = -10                   ' Hawaii-Aleutian Standard Time
   
  ' GPS unit commands
  GPS_GetInfo             = 0                     ' (2 bytes) Hardware, Firmware
  GPS_GetValid            = 1                     ' (1 byte)  0 = Not Valid, 1 = Valid
  GPS_GetSats             = 2                     ' (1 byte)
  GPS_GetTime             = 3                     ' (3 bytes) UTC, Hours, Minutes, Seconds
  GPS_GetDate             = 4                     ' (3 bytes) UTC, Day, Month, Year
  GPS_GetLat              = 5                     ' (5 bytes) Degrees, Minutes, Fractional Minutes (Word), Direction (0 = N, 1 = S)
  GPS_GetLong             = 6                     ' (5 bytes) Degrees, Minutes, Fractional Minutes (Word), Direction (0 = E, 1 = W) 
  GPS_GetAlt              = 7                     ' (word) in tenths of meters                                             
  GPS_GetSpeed            = 8                     ' (word) in tenths of knots
  GPS_GetHead             = 9                     ' (word) in tenths of degrees  

OBJ
  Math  : "Float32Full"
  F     : "FloatMath"  
  GPS   : "GPS_SmartMode"
  Ser   : "FullDuplexSerial" 

VAR

  long  rx, tx, baud                                    ' for the serial object
  long  _time_offset, _allow_DST
       
DAT
{
'Degrees Minutes Seconds Negate format
  D_LAT long 28,60,207,0
  D_LON long 81,19,897,1

  D_LAT long 28,35,4198,0
  D_LON long 81,12,2926,1

'Floating point degrees format
  D_LAT long 51.743357
  D_LON long -2.211916

  D_LAT long 28.601616
  D_LON long -81.198854
}
'UCF Engineering
  D_LAT long 28.602021
  D_LON long -81.198815
          
PUB stop
{{ Stop driver - free cogs }}
  Ser.stop
 
PUB start : Success
  stop
  DIRA[6]~~
  OUTA[6]~~


  '' Initialize the GPS Smart Mode Object
  _time_offset := EASTERN_TIME
  _allow_DST := TRUE
  
  ' Start the serial object
  Success := Ser.start(GPS_RX_PIN, GPS_RX_PIN, %1100, 4800)                        ' Start Full Duplex Serial Object


  'SmartStart                                            ' Wait until we connect to satillites
PUB SmartStart
  repeat while GetGPSByte(GPS_GetValid) == 0
    waitcnt(clkfreq + cnt)   

''Returns Destination as Float, in decimal degrees, converted from -8112.2926' format
PUB DestLongToFloat | DESTLONG
  DESTLONG := GetGPSFloat(D_LON[0],D_LON[1],D_LON[2],D_LON[3]) 
  return DESTLONG
  
PUB DestLatToFloat | DESTLAT
  DESTLAT := GetGPSFloat(D_LAT[0],D_LAT[1],D_LAT[2],D_LAT[3])
  return DESTLAT
   
''Returns Destination in degrees, already in -81.20884 format     
PUB DestLongInFloat | DESTLONG   
  DESTLONG := D_LON[0]
  return DESTLONG 

PUB DestLatInFloat |  DESTLAT
  DESTLAT :=  D_LAT[0]
  return DESTLAT


PUB GetGPSByte(cmd)
'' If we want to get more info from the GPS then this is what we call
'' Send a command to the GPS module and read a byte back
  Ser.str(string("!GPS"))                              ' Send GPS Command Header
  Ser.tx(cmd)                                          ' Send GPS Command
  return Ser.rx                                        ' Receive byte

''Returns Current Location in degrees
PUB GetLatitude
'' Return the latitude from the GPS modules as a single floating point value
  return GetXitude(GPS_GetLat)


PUB GetLongitude
'' Return the longitude from the GPS modules as a single floating point value
  return GetXitude(GPS_GetLong)
  
PRI GetXitude(cmd) | deg, mn, minFrac, negate
'' Get the lat/long-itude from the GPS unit
'' Return as a single floating point value   
  
  Ser.str(string("!GPS"))                              ' Send GPS Command Header
  Ser.tx(cmd)                                          ' Send GPS Command (GetLat)
  deg := Ser.rx                                        ' Receive 1st byte
  mn := Ser.rx                                         ' Receive 2nd byte
  minFrac := ReadWord                                  ' Receive 3rd, 4th bytes as word
  negate := Ser.rx==1                                  ' Receive 5th byte 
  return GetGPSFloat(deg, mn, minFrac, negate)
  
PRI ReadWord : aword
'' Read a word from the GPS module. Put the first value read into the high byte of the word
  aword.byte[1]  := Ser.rx                            'high byte transmitted first
  aword.byte[0]  := Ser.rx                            'then low byte into rightmost byte    
  
PUB GetGPSFloat(deg, mn, minFrac, negate) : single | minsF, ang
'' convert the given degrees, mins, and fractional minutes to a single floating point value
'' The negate flag converts from W to E or from N to S
  minsF := F.FAdd(F.FFloat(mn), F.FDiv(F.FFloat(minFrac), 10000.0)) 
  ang := F.FAdd(F.FFloat(deg), F.FDiv(minsF, 60.0))
  if negate
    ang := F.FNeg(ang)
  return ang